@@ -545,23 +545,18 @@ vips_icc_print_profile( const char *name, cmsHPROFILE profile )
 }
 #endif /*DEBUG*/
 
-/* How many bands we expect to see from an image after preprocessing by our
- * parent classes. This is a bit fragile :-( 
- *
- * FIXME ... split the _build() for colour into separate preprocess / process
- * / postprocess phases so we can load profiles after preprocess but before
- * actual processing takes place.
+/* TRUE if the number of bands in the ICC profile is compatible with the
+ * image.
  */
-static int
-vips_image_expected_bands( VipsImage *image )
+static gboolean
+vips_image_is_profile_compatible( VipsImage *image, int profile_bands )
 {
-	int expected_bands;
-
 	switch( image->Type ) {
 		case VIPS_INTERPRETATION_B_W:
 		case VIPS_INTERPRETATION_GREY16:
-			expected_bands = 1;
-			break;
+			/* The ICC profile needs to be monochrome.
+			 */
+			return( profile_bands == 1 );
 
 		case VIPS_INTERPRETATION_XYZ:
 		case VIPS_INTERPRETATION_LAB:
@@ -575,22 +570,24 @@ vips_image_expected_bands( VipsImage *image )
 		case VIPS_INTERPRETATION_RGB16:
 		case VIPS_INTERPRETATION_scRGB:
 		case VIPS_INTERPRETATION_HSV:
-			expected_bands = 3;
-			break;
+			/* The band count in the ICC profile must correspond to that of
+			 * the image, with a maximum of 3 bands allowed.
+			 */
+			return( VIPS_MIN( 3, image->Bands ) == profile_bands );
+
+		case VIPS_INTERPRETATION_CMYK:
+			/* CMYK images can only be imported if the ICC-profile has at
+			 * least 4 bands thereby blocking the usage of RGB profiles.
+			 */
+			return( profile_bands >= 4 );
 
 		case VIPS_INTERPRETATION_MULTIBAND:
 		case VIPS_INTERPRETATION_HISTOGRAM:
-		case VIPS_INTERPRETATION_CMYK:
 		case VIPS_INTERPRETATION_MATRIX:
 		case VIPS_INTERPRETATION_FOURIER:
 		default:
-			expected_bands = image->Bands;
-			break;
+			return( image->Bands >= profile_bands );
 	}
-
-	expected_bands = VIPS_MIN( expected_bands, image->Bands );
-
-	return( expected_bands );
 }
 
 /* Load a profile from a blob and check compatibility with image, intent and
@@ -631,7 +628,7 @@ vips_icc_load_profile_blob( VipsBlob *blob,
         }
 
 	if( image &&
-		vips_image_expected_bands( image ) != info->bands ) { 
+		!vips_image_is_profile_compatible( image, info->bands ) ) {
 		VIPS_FREEF( cmsCloseProfile, profile );
 		g_warning( "%s", _( "profile incompatible with image" ) );
 		return( NULL );
@@ -720,8 +717,22 @@ vips_icc_set_import( VipsIcc *icc,
 	 */
 	if( code->in &&
 		!icc->in_profile ) {
-		const char *name = code->in->Type == VIPS_INTERPRETATION_CMYK ?
-			"cmyk" : "srgb";
+		const char *name;
+
+		switch( code->in->Type ) {
+		case VIPS_INTERPRETATION_B_W:
+		case VIPS_INTERPRETATION_GREY16:
+			name = "sgrey";
+			break;
+
+		case VIPS_INTERPRETATION_CMYK:
+			name = "cmyk";	
+			break;
+
+		default:
+			name = "srgb";
+			break;
+		}
 
 		if( 
 			!vips_profile_load( name, &icc->in_blob, NULL ) &&
@@ -856,9 +867,10 @@ decode_lab( guint16 *fixed, float *lab, int n )
 	}
 }
 
-#define X_FAC (cmsD50X / VIPS_D65_X0)
-#define Y_FAC (cmsD50Y / VIPS_D65_Y0)
-#define Z_FAC (cmsD50Z / VIPS_D65_Z0)
+/* The matrix already includes the D65 channel weighting, so we just scale by
+ * Y.
+ */
+#define SCALE (VIPS_D65_Y0)
 
 static void
 decode_xyz( guint16 *fixed, float *xyz, int n )
@@ -868,9 +880,27 @@ decode_xyz( guint16 *fixed, float *xyz, int n )
 	for( i = 0; i < n; i++ ) {
 		/* cmsXYZEncoded2Float inlined.
  		 */
-		xyz[0] = (double) fixed[0] / (X_FAC * 32768.0);
-		xyz[1] = (double) fixed[1] / (Y_FAC * 32768.0);
-		xyz[2] = (double) fixed[2] / (Z_FAC * 32768.0);
+		float X = fixed[0] / 32768.0;
+		float Y = fixed[1] / 32768.0;
+		float Z = fixed[2] / 32768.0;
+
+		X *= SCALE;
+		Y *= SCALE;
+		Z *= SCALE;
+
+		/* Transform XYZ D50 to D65, chromatic adaption is done with the
+		 * Bradford transformation.
+		 * See: https://fujiwaratko.sakura.ne.jp/infosci/colorspace/bradford_e.html
+		 */
+		xyz[0] = 0.955513 * X +
+			-0.023073 * Y +
+			0.063309 * Z;
+		xyz[1] = -0.028325 * X +
+			1.009942 * Y +
+			0.021055 * Z;
+		xyz[2] = 0.012329 * X +
+			-0.020536 * Y +
+			1.330714 * Z;
 
 		xyz += 3;
 		fixed += 3;
@@ -905,7 +935,11 @@ vips_icc_import_line( VipsColour *colour,
 		else
 			decode_xyz( encoded, q, chunk );
 
-		p += PIXEL_BUFFER_SIZE * VIPS_IMAGE_SIZEOF_PEL( colour->in[0] );
+		// use input_bands, since in[0] may have had alpha removed,
+		// and can have 1, 3 or 4 bands
+		p += PIXEL_BUFFER_SIZE * 
+			colour->input_bands * 
+			VIPS_IMAGE_SIZEOF_ELEMENT( colour->in[0] );
 		q += PIXEL_BUFFER_SIZE * 3;
 	}
 }
@@ -1015,9 +1049,23 @@ encode_xyz( float *in, float *out, int n )
 	int i;
 
 	for( i = 0; i < n; i++ ) {
-		out[0] = in[0] * X_FAC;
-		out[1] = in[1] * Y_FAC;
-		out[2] = in[2] * Z_FAC;
+		float X = in[0] / SCALE;
+		float Y = in[1] / SCALE;
+		float Z = in[2] / SCALE;
+
+		/* Transform XYZ D65 to D50, chromatic adaption is done with the
+		 * Bradford transformation.
+		 * See: https://fujiwaratko.sakura.ne.jp/infosci/colorspace/bradford_e.html
+		 */
+		out[0] = 1.047886 * X +
+			0.022919 * Y +
+			-0.050216 * Z;
+		out[1] = 0.029582 * X +
+			0.990484 * Y +
+			-0.017079 * Z;
+		out[2] = -0.009252 * X +
+			0.015073 * Y +
+			0.751678 * Z;
 
 		in += 3;
 		out += 3;
@@ -1034,7 +1082,7 @@ vips_icc_export_line_xyz( VipsColour *colour,
 	VipsPel *q;
 	int x;
 
-	/* Buffer of encoded float pixels we transform.
+	/* Buffer of encoded float pixels we transform to device space.
 	 */
 	float encoded[3 * PIXEL_BUFFER_SIZE];
 
@@ -1047,7 +1095,11 @@ vips_icc_export_line_xyz( VipsColour *colour,
 		cmsDoTransform( icc->trans, encoded, q, chunk );
 
 		p += PIXEL_BUFFER_SIZE * 3;
-		q += PIXEL_BUFFER_SIZE * VIPS_IMAGE_SIZEOF_PEL( colour->out );
+		// use colour->bands, since out may have had alpha reattached
+		// and can have extra bands
+		q += PIXEL_BUFFER_SIZE * 
+			colour->bands *
+			VIPS_IMAGE_SIZEOF_ELEMENT( colour->out );
 	}
 }
 
@@ -1318,7 +1370,7 @@ vips_icc_is_compatible_profile( VipsImage *image,
                 return( FALSE );
         }
 
-	if( vips_image_expected_bands( image ) != info->bands ) {
+	if( !vips_image_is_profile_compatible( image, info->bands ) ) {
 		/* Bands mismatch.
 		 */
 		VIPS_FREEF( cmsCloseProfile, profile );
@@ -1377,7 +1429,7 @@ vips_icc_is_compatible_profile( VipsImage *image,
  *
  * If @embedded is set, the input profile is taken from the input image
  * metadata. If there is no embedded profile,
- * @input_profile_filename is used as a fall-back. 
+ * @input_profile is used as a fall-back. 
  * You can test for the
  * presence of an embedded profile with
  * vips_image_get_typeof() with #VIPS_META_ICC_NAME as an argument. This will
