@@ -426,9 +426,11 @@ vips_foreign_save_heif_write( struct heif_context *ctx,
 
 	struct heif_error error;
 
-	error.code = 0;
-	if( vips_target_write( heif->target, data, length ) )
-		error.code = -1;
+	error.code = heif_error_Ok;
+	if( vips_target_write( heif->target, data, length ) ) {
+		error.code = heif_error_Encoding_error;
+		error.subcode = heif_suberror_Cannot_write_output_data;
+	}
 
 	return( error );
 }
@@ -545,6 +547,11 @@ vips_foreign_save_heif_build( VipsObject *object )
 	heif->page_height = vips_image_get_page_height( heif->image );
 	heif->n_pages = heif->image->Ysize / heif->page_height;
 
+	if( heif->page_width > 16384 || heif->page_height > 16384 ) {
+		vips_error( "heifsave", _( "image too large" ) );
+		return( -1 );
+	}
+
 	/* Make a heif image the size of a page. We send sink_disc() output 
 	 * here and write a frame each time it fills.
 	 */
