@@ -249,7 +249,7 @@ vips_foreign_save_jxl_build( VipsObject *object )
 	if( !vips_object_argument_isset( object, "distance" ) ) 
 		jxl->distance = jxl->Q >= 30 ?
 			0.1 + (100 - jxl->Q) * 0.09 :
-			6.24 + pow(2.5, (30 - jxl->Q) / 5.0f) / 6.25f;
+			53.0 / 3000.0 * jxl->Q * jxl->Q - 23.0 / 20.0 * jxl->Q + 25.0;
 
 	/* Distance 0 is lossless. libjxl will fail for lossy distance 0.
 	 */
@@ -313,13 +313,13 @@ vips_foreign_save_jxl_build( VipsObject *object )
 	switch( in->Type ) {
 	case VIPS_INTERPRETATION_B_W:
 	case VIPS_INTERPRETATION_GREY16:
-		jxl->info.num_color_channels = 1;
+		jxl->info.num_color_channels = VIPS_MIN( 1, in->Bands );
 		break;
 
 	case VIPS_INTERPRETATION_sRGB:
 	case VIPS_INTERPRETATION_scRGB:
 	case VIPS_INTERPRETATION_RGB16:
-		jxl->info.num_color_channels = 3;
+		jxl->info.num_color_channels = VIPS_MIN( 3, in->Bands );
 		break;
 
 	default:
@@ -542,7 +542,7 @@ vips_foreign_save_jxl_class_init( VipsForeignSaveJxlClass *class )
 		_( "Target butteraugli distance" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveJxl, distance ),
-		0, 15, 1.0 );
+		0.0, 25.0, 1.0 );
 
 	VIPS_ARG_INT( class, "effort", 12, 
 		_( "Effort" ), 
